@@ -15,6 +15,10 @@ from gnosis.safe.multi_send import MultiSend, MultiSendOperation, MultiSendTx
 from gnosis.safe.safe_tx import SafeTx
 
 
+MULTISEND_CALL_ONLY = '0x40A2aCCbd92BCA938b02010E17A5b8929b49130D'
+multisends = {
+    250: '0x10B62CC1E8D9a9f1Ad05BCC491A7984697c19f7E',
+}
 transaction_service = {
     1: 'https://safe-transaction.mainnet.gnosis.io',
     4: 'https://safe-transaction.rinkeby.gnosis.io',
@@ -22,6 +26,7 @@ transaction_service = {
     56: 'https://safe-transaction.bsc.gnosis.io',
     100: 'https://safe-transaction.xdai.gnosis.io',
     137: 'https://safe-transaction.polygon.gnosis.io',
+    250: 'https://safe.fantom.network',
     246: 'https://safe-transaction.ewc.gnosis.io',
     42161: 'https://safe-transaction.arbitrum.gnosis.io',
     73799: 'https://safe-transaction.volta.gnosis.io',
@@ -37,17 +42,16 @@ class ApiError(Exception):
 
 
 class ApeSafe(Safe):
-    base_url = 'https://safe-transaction.mainnet.gnosis.io'
-    multisend = '0x40A2aCCbd92BCA938b02010E17A5b8929b49130D'
 
-    def __init__(self, address):
+    def __init__(self, address, base_url=None, multisend=None):
         """
         Create an ApeSafe from an address or a ENS name and use a default connection.
         """
         if not web3.isChecksumAddress(address):
             address = web3.ens.resolve(address)
         ethereum_client = EthereumClient(web3.provider.endpoint_uri)
-        self.base_url = transaction_service[chain.id]
+        self.base_url = base_url or transaction_service[chain.id]
+        self.multisend = multisend or multisends.get(chain.id, MULTISEND_CALL_ONLY)
         super().__init__(address, ethereum_client)
 
     def __str__(self):
@@ -153,7 +157,7 @@ class ApeSafe(Safe):
         """
         return self.estimate_tx_gas(safe_tx.to, safe_tx.value, safe_tx.data, safe_tx.operation)
 
-    def preview(self, safe_tx: SafeTx, events=True, call_trace=False, reset=True):
+    def preview(self, safe_tx: SafeTx, events=True, call_trace=False, reset=True, gas_limit=None):
         """
         Dry run a Safe transaction in a forked network environment.
         """
@@ -161,13 +165,15 @@ class ApeSafe(Safe):
             chain.reset()
         tx = copy(safe_tx)
         safe = Contract.from_abi('Gnosis Safe', self.address, self.get_contract().abi)
-        # replace pending nonce with the subsequent nonce
+        # Replace pending nonce with the subsequent nonce
         tx.safe_nonce = safe.nonce()
         # Forge signatures from the needed amount of owners, skip the one which submits the tx
-        owners = [accounts.at(owner, force=True) for owner in sorted(safe.getOwners())]
+        # Owners must be sorted numerically, sorting as checksum addresses may yield wrong order
+        owners = [accounts.at(owner, force=True) for owner in sorted(safe.getOwners(), key=str.lower)]
         threshold = safe.getThreshold()
         for owner in owners[1:threshold]:
-            safe.approveHash(tx.safe_tx_hash.hex(), {'from': owner})
+            safe.approveHash(tx.safe_tx_hash.hex(), {'from': owner, 'gas_price': 0, 'gas_limit': gas_limit})
+
         # Signautres are encoded as [bytes32 r, bytes32 s, bytes8 v]
         # Pre-validated signatures are encoded as r=owner, s unused and v=1.
         # https://docs.gnosis.io/safe/docs/contracts_signatures/#pre-validated-signatures
@@ -185,7 +191,7 @@ class ApeSafe(Safe):
             signatures,
         ]
 
-        receipt = safe.execTransaction(*args, {'from': owners[0]})
+        receipt = safe.execTransaction(*args, {'from': owners[0], 'gas_price': 0, 'gas_limit': gas_limit})
         if 'ExecutionSuccess' not in receipt.events:
             receipt.info()
             receipt.call_trace(True)
@@ -197,6 +203,11 @@ class ApeSafe(Safe):
         if call_trace:
             receipt.call_trace(True)
 
+        # Offset gas refund for clearing storage when on-chain signatures are consumed.
+        # https://github.com/gnosis/safe-contracts/blob/v1.1.1/contracts/GnosisSafe.sol#L140
+        refunded_gas = 15_000 * (threshold - 1)
+        click.secho(f'recommended gas limit: {receipt.gas_used + refunded_gas}', fg='green', bold=True)
+
         return receipt
 
     def preview_pending(self, events=True, call_trace=False):
